/*
   ICESTAT.C - iceStat v1.2
   Copyright (C) 1996 by Albert Vernon and Joshua Hines.
   You may use any of this code that you wish, as long as you
   give credit to us.  If you use the spin function,
   you must give credit to Jon Rickher.  If you use any of the
   multitasker aware functions, you must give credit to Wayne Bell.

   Most of this code is ANSI C compatible.
   It will compile successfully under Borland's Turbo C++ v3.00
   for DOS.  You may have to make minor modifications for other
   compilers.

   last modification:  03 July 1996
*/

#pragma warn -aus

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <time.h>
#include <dos.h>

/* vardec.h is a header file from the WWIV software by Wayne Bell.
   You need to include a copy of vardec.h from WWIV version 4.22
   or greater.  Versions v4.23+ may require additional header files.
   Read the compiler's error message to find out which one(s).
*/
#include "vardec.h"


userrec thisuser;  /* userrec data structure defined in vardec.h */
char    ansi,            /* ansi enabled? */
		data[81],        /* data dir with trailing backslash */
		systemname[51];
int     usernum,
		multitasker;
void    show_help (void),
		goxy (int x, int y),
		left (int amount),
		display_data (void),
		spin (unsigned char *s),
		detect_multitask (void),
		giveup_timeslice (void),
		win_pause (void),
		dv_pause (void);
int     test_ansi (char *filename),
		display (char *filename),
		read_chaintxt (char *filename),
		read_userlst (void),
		get_dos_version (void),
		get_win_version (void),
		get_dv_version (void);

FILE    *chaintxt,
		*userlst;

/*=========================================================================*/

void main (int argc, char *argv[])
{

	if (argc < 2)
	{
		show_help();
		exit (1);
	}

	if (!test_ansi(argv[1])) /* if they don't have ANSI, simply exit */
		exit (1);

	detect_multitask();

    printf ("\n\x1b[0;31mܱ \x1b[0;1;31miceStat\x1b[0;1;33m:\x1b[0;31m \x1b[0;1mWould you like to view your statistics? \x1b[0;31m\x1b[0;1;33m (\x1b[0;1my\x1b[0;1;33m/\x1b[0;1mN[0;1;33m): ");
	if ( !noyes() )
	{
		puts ("\n");
		exit (0);
	}
	if (!read_chaintxt(argv[1]))
	{
		printf ("\n## Error:  unable to read data from %s. \n", argv[1]);
		exit (2);
	}

	if (!read_userlst())
	{
		printf ("\n## Error:  unable to read data from %sUSER.LST. \n", data);
		exit (2);
	}

	printf ("\x1b[2J"); /* clear screen */
	if (!display ("ICESTAT.ANS"))
	{
		printf ("\n## Error:  unable to display ICESTAT.ANS. \n");
		exit (2);
	}

	display_data ();
	printf ("\x1b[2J"); /* clear screen */
}

/*===========================================================================
 goxy moves the cursor to a specific point on the screen using
   ANSI escape sequences.
===========================================================================*/

void goxy(int x, int y)
{
	printf ("\x1b[%d;%dH", y, x);
}

/*===========================================================================
 left uses ANSI escape sequences to cause the cursor to move left a
   specified amount.
===========================================================================*/

void left(int amount)
{
	printf ("\x1b[%dD", amount);
}

/*===========================================================================
 test_ansi examines CHAIN.TXT to determine if the user has ANSI graphics
   support turned on.
===========================================================================*/

int test_ansi (char *filename)
{
	char    buffer[81],
			ansi_status;
	register int i;

	if ((chaintxt = fopen(filename, "rt")) == NULL)
	  {
		printf ("\n## Error:  unable to read data from %s. \n", filename);
		exit (2);
	  }

	for (i = 1; i < 14; i++)   /* this loop moves to line #14 */
		fgets (buffer, 80, chaintxt);
	ansi_status = getc (chaintxt);
	fclose (chaintxt);
	if (ansi_status != '0')
		return 1;
	else
		return 0;
}

/*=========================================================================*/

void show_help (void)
{
    printf ("\n\niceStat v1.2 \n");
    printf ("Copyright (C) 1996 by Albert Vernon & Joshua Hines\n");
	printf ("Beta testing by Troy Lancaster, Jim Nunn, & Kyle Miller. \n");
	printf ("Additional thanks to Wayne Bell & Jon Rickher. \n");
	printf ("Compiled under Turbo C++ v3.00 on %s \n", __DATE__);
	printf ("Syntax as a logon event or chain is: \n");
	printf ("    ICESTAT %1 \n");
	printf ("This program is FREEWARE and may be freely used and copied. \n");
}

/*===========================================================================
display outputs the contents of a specificed file to standard output
   device by block reading the file into memory, then block writing the
   memory buffer to stdout.
===========================================================================*/

int     display (char *filename)
{
	FILE    *fp;
	struct  stat    buff;
	char    *membuf;

	if ((fp = fopen(filename, "rb")) == NULL)
	  return(0);

	stat(filename, &buff);

	if ((membuf = malloc(buff.st_size + 1024)) == NULL) {
	  printf("\ncannot allocate memory\n");
	  fclose(fp);
	  return(0);
	}

	fread( (void *)membuf, buff.st_size, 1, fp);
	fwrite( (void *)membuf, buff.st_size, 1, stdout);
	fclose(fp);
	free(membuf);
	return(1);
}

/*===========================================================================
 read_chaintxt reads in data from specific lines of the specified
   file name.
===========================================================================*/

int  read_chaintxt (char *filename)
{
	char    temp[7];
	int     i;

	if ((chaintxt = fopen(filename, "rt")) == NULL)
	  return (0);

	fgets (temp, 6, chaintxt);   /* grab first line and */
	usernum = atoi (temp);       /* put it into usernum */

	for (i = 1; i < 18; i++)                 /* grab data dir off line #18 */
		fgets (data, 81, chaintxt);
	data[strlen (data) - 1] = '\0';

	for (i = 18; i < 22; i++)  /* grab system name off line #22 */
		fgets (systemname, 51, chaintxt);
	systemname[strlen (systemname) - 1] = '\0';

	fclose (chaintxt);
	return (1);
}
/*=========================================================================*/

int  read_userlst (void)
{
	char    path[128];
	long    size;

	size = sizeof (userrec);

	sprintf (path, "%sUSER.LST", data);
	if ((userlst = fopen (path, "rb")) == NULL)
	  return (0);

	fseek (userlst, size * (long) (usernum), SEEK_SET);
	fread (&thisuser, size, 1, userlst);

	fclose (userlst);
	return (1);
}

/*=========================================================================*/

void    display_data (void)
{
	time_t now;  /* holds time data */
	int throwaway;

	time (&now);  /* fill structure with current time info */


    printf ("\x1b[1;37;44m");  /* set color to white on blue */
    goxy (44, 4);  printf ("%s #%u", thisuser.name, usernum);
    printf ("\x1b[1;37;40m");  /* set color to white on black */

    goxy (12, 10); printf ("%s", ctime (&now));
    goxy (19, 11); printf (systemname);
    goxy (17, 13); printf (thisuser.city);
    goxy (18, 14); printf ("%u bps", thisuser.lastrate);
    goxy (22, 15); printf ("%.0f minutes", thisuser.timeon / 60);
    goxy (19, 16); printf ("%u", thisuser.sl);
    goxy (28, 17); printf ("%u", thisuser.dsl);
    goxy (8, 18);  printf ("%c", thisuser.sex);
    goxy (8, 19);  printf ("%u", thisuser.age);
    goxy (15, 20); printf ("%s", thisuser.laston);
    goxy (17, 21); printf ("%u", thisuser.waiting);
    goxy (67, 13); printf ("%u", thisuser.msgpost);
    goxy (66, 14); printf ("%u", thisuser.uploaded);
    goxy (66, 15); printf ("%luK", thisuser.uk);
    goxy (68, 16); printf ("%u", thisuser.downloaded);
    goxy (68, 17); printf ("%luK", thisuser.dk);
    goxy (63, 18); printf ("%u", thisuser.logons);
    goxy (62, 19); printf ("%lu", (unsigned long) (thisuser.emailsent
                                  + thisuser.emailnet
                                  + thisuser.feedbacksent) );
    goxy (52, 21);
    if (thisuser.forwardusr)
		printf ("%u@%u", thisuser.forwardusr, thisuser.forwardsys);
	else
		printf ("<not forwarded>");

	goxy (35, 22);
	while ( !kbhit() )
	{
		printf ("\x1b[1;34;40mPRESS A KEY");  /* switch color to blue */
		left (11);
		spin ("PRESS A KEY");
		left (11);
	}
	throwaway = getch();
    printf ("\x1b[0m");  /* reset colors */
}

/*===========================================================================
   spin is a slightly modified version of Jon Rickher's spin function from
   his ENHANCE.C.  If you use this function, you >must< give credit
   to Jon Rickher.
===========================================================================*/

void spin (unsigned char *s)
{
	register int i = 0,
				 dly = 18;

	printf ("\x1b[1;36;40m");  /* switch color to cyan */
	while (s[i])
	{
		/* this loop runs inside of a loop which is controlled by the
		   kbhit() function.  The kbhit() function is VERY multitasker
		   unfriendly.  The giveup_timeslice function compensates for
		   that.  Otherwise systems running a multitasker would come
		   to a grinding halt whenever the spinning cursor was
		   being displayed.  */

		giveup_timeslice();
		delay(dly); putchar('/');  left(1);
		delay(dly); putchar('-');  left(1);
		delay(dly); putchar('\\'); left(1);
		delay(dly); putchar('|');  left(1);
		delay(dly); putchar(s[i]); delay(dly);
		i++;
	}
}

/*===========================================================================
   noyes returns 1 if a YES response is indicated, 0 if a NO response
   is indicated.  A CR counts as a NO response.
===========================================================================*/

int noyes (void)
{
	int inchar;

repeat:
	giveup_timeslice();
	inchar = toupper (getch());
	if (inchar == 'Y')
		return 1;
	else if ( (inchar == 'N') || (inchar == '\r') )
		return 0;
	else goto repeat;
}

/*===========================================================================
The code from here to the end of the file is copyright by Wayne Bell,
the author of WWIV BBS software.  Wayne has granted permission for these
functions to be freely used and distributed.  So you are welcome to
use them in any of your programs, but you must give him the appropriate
credit.
===========================================================================*/

#define MT_DESQVIEW 0x01
#define MT_WINDOWS  0x02
#define MT_OS2      0x04


/* 
   This function must be called at the beginning of your program,
   before you first call giveup_timeslice() .
*/

void detect_multitask(void)
{
  get_dos_version();
  get_win_version();
  get_dv_version();
}

/*=========================================================================*/

int get_dos_version(void)
{
  _AX=0x3000;
  geninterrupt(0x21);
  if(_AX%256 >=10) {
	multitasker |= MT_OS2;
  }
  return(_AX);
}

/*=========================================================================*/

int get_dv_version(void)
{
  int v;

  _AX=0x2b01;
  _CX=0x4445;
  _DX=0x5351;
  geninterrupt(0x21);
  if (_AL == 0xff) {
	return 0;
  } else {
	v=_BX;
	multitasker |= MT_DESQVIEW;
	return v;
  }
}

/*=========================================================================*/

int get_win_version(void)
{
  int v=0;

/*
 *  push  bp
 *  push  es
 *  push  bx
 */
  __emit__(0x55, 0x06, 0x53);

  _AX=0x352f;
  geninterrupt(0x21);
  _AX=_ES;

  if (_AX | _BX) {

	_AX=0x1600;
	geninterrupt(0x2f);

	v=_AX;
	if (v%256<=1)
	  v=0;
  }

/*
 * pop bx
 * pop es
 * pop bp
 */
  __emit__(0x5b, 0x07, 0x5d);

  if (v!=0)
	multitasker |= MT_WINDOWS;
  return(v);
}

/*=========================================================================*/

void giveup_timeslice(void)
{
  if (multitasker) {
	switch (multitasker) {
	  case 1 : /* DESQView pause */
		  dv_pause();
		  break;
	  case 2 : /* Windows pause */
		  win_pause();
		  break;
	  case 3 : /* Win or DV pause */
		  win_pause();
		  break;
	  case 4:  /* OS/2 pause */
		  _AX=0x1680;
		  geninterrupt(0x2f);
		  break;
	  case 5:
	  case 6:
	  case 7:
		  delay(17);
		  win_pause();
		  break;
	  default: 
		  puts ("Unrecognized multitasker");
		  break;
	}
  }
}

/*=========================================================================*/

void dv_pause(void)
{
  _AX=0x101a;
  geninterrupt(0x15);
  _AX=0x1000;
  geninterrupt(0x15);
  _AX=0x1025;
  geninterrupt(0x15);
}

/*=========================================================================*/

void win_pause(void)
{
  __emit__(0x55, 0xb8, 0x80, 0x16, 0xcd, 0x2f, 0x5d);
/*
 * push bp
 * mov ax,0x1680
 * int 0x2f
 * pop bp
 */
}

/*==END OF FILE ===========================================================*/


